define('cache', function (require, _exports, module) {
  function Cache(allowOverride) {
    this.data = {};
    this.allowOverride = allowOverride;
  }
  Cache.prototype.put = function (key, fn) {
    if (key in this.data && !this.allowOverride)
      throw 'Key {' + key + '} already exists!';
    this.data[key] = fn;
  };
  Cache.prototype.get = function (key) {
    var data = this.data;
    return new Promise(function (resolve, reject) {
      if (key in data) return resolve(data[key]);
      var xhr = new XMLHttpRequest();
      xhr.open('GET', key, true);
      xhr.onload = function () {
        resolve(data[key] = _.template(this.responseText, {variable: 'it'}));
      };
      xhr.onerror = function () {
        reject(this);
      };
      xhr.send();
    });
  };

  var cache = module.exports = new Cache();
  require('templates');

  var BaseView = cache.BaseView = Backbone.View.extend({
    initialize: function () {
      var _this = this;
      _this.subviews = {data: {}};
      _this.childViews = [];
      if (_this.templateUrl) {
        _this.__gotTemplate = cache.get(_this.templateUrl)
        .then(function (fn) {
          _this.templateFn = fn;
        });
      }
      _.bindAll(_this, 'render', 'postrender');
      _this.render();
    },
    clear: function () {
      var _this = this;
      if (_this.childViews.length) {
        _this.childViews.forEach(function (view) {
          view.remove();
        });
        _this.childViews = [];
      }
    },
    remove: function () {
      var _this = this;
      _this.clear();
      _this.undelegateEvents();
      Backbone.View.prototype.remove.call(_this);
    },
    _render: function () {
      this.$el.html(this.templateFn());
    },
    render: function () {
      var _this = this;
      (_this.__gotTemplate || Promise.resolve())
        .then(_this._render.bind(_this))
        .then(_this.postrender);
      return _this;
    },
    postrender: function () {
      _.forEach(this.$('[data-i18n]'), function (node) {
        node.innerHTML = _.i18n(node.dataset.i18n);
      });
    },
    loadSubview: function (name, factory, selector) {
      var _this = this;
      var view;
      var subviews = _this.subviews;
      view = subviews.data[name];
      if (!view) {
        view = factory();
      }
      var current = subviews.current;
      if (name !== current) {
        var currentView = subviews.data[current];
        if (currentView) {
          currentView.remove();
          subviews.data[current] = null;
        }
      }
      subviews.data[subviews.current = name] = view;
      var $el = selector ? _this.$(selector) : _this.$el;
      $el.html(view.render().el);
      return view;
    },
    getValue: function (target) {
      var key = target.dataset.id;
      var value;
      switch (key[0]) {
      case '!':
        key = key.slice(1);
        value = target.checked;
        break;
      case '[':
        key = key.slice(1);
        value = _.filter(target.value.split('\n').map(function (s) {return s.trim();}));
        break;
      default:
        value = target.value;
      }
      return {
        key: key,
        value: value,
      };
    },
  });

  BaseView.prototype.postrender.call(window);

  cache.BaseRouter = Backbone.Router.extend({
    initialize: function (selector) {
      var _this = this;
      _this.views = {data: {}};
      _this.$root = $(selector);
    },
    loadView: function (name, factory) {
      var _this = this;
      var views = _this.views;
      var view = views.data[name];
      if (!view) view = views.data[name] = factory();
      if (name !== views.current) {
        var currentView = views.data[views.current];
        if (currentView) {
          currentView.remove();
          views.data[views.current] = null;
        }
        views.data[views.current = name] = view;
      }
      _this.$root.html(view.el);
      return view;
    },
  });

  !function () {
    var xhr = new XMLHttpRequest;
    xhr.open('GET', '/images/sprite.svg', true);
    xhr.onload = function () {
      var div = document.createElement('div');
      div.style.display = 'none';
      div.innerHTML = xhr.responseText;
      document.body.insertBefore(div, document.body.firstChild);
    };
    xhr.send();
  }();
});

/* Templates cached from `_.template` with love :) */

define('templates', function (require, exports, module) {
  var cache = require('cache');
cache.put("/options/templates/confirm-options.html", function(it){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<label> <input type="checkbox" data-check="closeAfterInstall" id="cbClose" '+
((__t=(
  it.closeAfterInstall ? 'checked' : '' ))==null?'':__t)+
'> <span data-i18n="installOptionClose"></span> </label> <label> <input type="checkbox" data-check="trackLocalFile" id="cbTrack" '+
((__t=(
  it.trackLocalFile ? 'checked' : '' ))==null?'':__t)+
' '+
((__t=(
  it.closeAfterInstall ? 'disabled' : '' ))==null?'':__t)+
'> <span data-i18n="installOptionTrack"></span> </label>';
return __p;
});
cache.put("/options/templates/confirm.html", function(it){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<div class="frame-header"> <div class="buttons"> <div> <button class="button-toggle" data-i18n="buttonInstallOptions"></button> </div> <button id="btnInstall" data-i18n="buttonConfirmInstallation"></button> <button id="btnClose" data-i18n="buttonClose"></button> </div> <h1><span data-i18n="labelInstall"></span> - <span data-i18n="extName"></span></h1> <div id="url" class="ellipsis"></div> <div id="msg" class="ellipsis"></div> </div> <div class="frame-body"> <div class="editor-code"></div> </div>';
return __p;
});
cache.put("/options/templates/edit-meta.html", function(it){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<table> <tr> <td title="@name" data-i18n="labelName"></td> <td class="expand"> <input type="text" data-id="name" value="'+
((__t=( it.name ))==null?'':_.escape(__t))+
'" placeholder="'+
((__t=( it.__name ))==null?'':_.escape(__t))+
'"> </td> <td title="@run-at" data-i18n="labelRunAt"></td> <td> <select data-id="run-at" value="'+
((__t=( it['run-at'] || 'default' ))==null?'':__t)+
'"> <option value="default" data-i18n="labelRunAtDefault"></option> <option value="start">document-start</option> <option value="idle">document-idle</option> <option value="end">document-end</option> </select> </td> </tr> <tr title="@homepageURL"> <td data-i18n="labelHomepageURL"></td> <td colspan="3" class="expand"> <input type="text" data-id="homepageURL" value="'+
((__t=( it.homepageURL ))==null?'':_.escape(__t))+
'" placeholder="'+
((__t=( it.__homepageURL ))==null?'':_.escape(__t))+
'"> </td> </tr> </table> <table> <tr title="@updateURL"> <td data-i18n="labelUpdateURL"></td> <td class="expand"> <input type="text" data-id="updateURL" value="'+
((__t=( it.updateURL ))==null?'':_.escape(__t))+
'" placeholder="'+
((__t=( it.__updateURL ))==null?'':_.escape(__t))+
'"> </td> </tr> <tr title="@downloadURL"> <td data-i18n="labelDownloadURL"></td> <td class="expand"> <input type="text" data-id="downloadURL" value="'+
((__t=( it.downloadURL ))==null?'':_.escape(__t))+
'" placeholder="'+
((__t=( it.__downloadURL ))==null?'':_.escape(__t))+
'"> </td> </tr> </table> <fieldset title="@include"> <legend> <span data-i18n="labelInclude"></span> <label> <input type="checkbox" data-id="!_include" '+
((__t=( it._include !== false ? 'checked' : '' ))==null?'':__t)+
'> <span data-i18n="labelKeepInclude"></span> </label> </legend> <div data-i18n="labelCustomInclude"></div> <textarea data-id="[include">'+
((__t=( (it.include || []).join('\n') ))==null?'':_.escape(__t))+
'</textarea> </fieldset> <fieldset title="@match"> <legend> <span data-i18n="labelMatch"></span> <label> <input type="checkbox" data-id="!_match" '+
((__t=( it._match !== false ? 'checked' : '' ))==null?'':__t)+
'> <span data-i18n="labelKeepMatch"></span> </label> </legend> <div data-i18n="labelCustomMatch"></div> <textarea data-id="[match">'+
((__t=( (it.match || []).join('\n') ))==null?'':_.escape(__t))+
'</textarea> </fieldset> <fieldset title="@exclude"> <legend> <span data-i18n="labelExclude"></span> <label> <input type="checkbox" data-id="!_exclude" '+
((__t=( it._exclude !== false ? 'checked' : '' ))==null?'':__t)+
'> <span data-i18n="labelKeepExclude"></span> </label> </legend> <div data-i18n="labelCustomExclude"></div> <textarea data-id="[exclude">'+
((__t=( (it.exclude || []).join('\n') ))==null?'':_.escape(__t))+
'</textarea> </fieldset>';
return __p;
});
cache.put("/options/templates/edit.html", function(it){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<div class="frame-header"> <div class="buttons"> <div> <button class="button-toggle" data-i18n="buttonCustomMeta"></button> </div> </div> <h2 data-i18n="labelScriptEditor"></h2> </div> <div class="frame-footer"> <div class="pull-right"> <button id="editorSave" data-i18n="buttonSave"></button> <button id="editorSaveClose" data-i18n="buttonSaveClose"></button> <button id="editorClose" data-i18n="buttonClose"></button> </div> <label> <input type="checkbox" data-id="!update" '+
((__t=( it.update ? 'checked' : '' ))==null?'':__t)+
'> <span data-i18n="labelAllowUpdate"></span> </label> </div> <div class="frame-body"> <div class="editor-code"></div> </div>';
return __p;
});
cache.put("/options/templates/main.html", function(it){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<aside> <img src="/images/icon128.png"> <h2 data-i18n="extName"></h2> <div class="line">2013-2016</div> <hr> <div class="sidemenu"> <a href="#main/installed" '+
((__t=( it.tab == 'main' ? 'class="active"' : ''  ))==null?'':__t)+
' data-i18n="sideMenuInstalled"></a> <a href="#main/settings" '+
((__t=( it.tab == 'settings' ? 'class="active"' : '' ))==null?'':__t)+
' data-i18n="sideMenuSettings"></a> <a href="#main/about" '+
((__t=( it.tab == 'about' ? 'class="active"' : '' ))==null?'':__t)+
' data-i18n="sideMenuAbout"></a> </div> </aside> <div id="tab"></div>';
return __p;
});
cache.put("/options/templates/message.html", function(it){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<div>'+
((__t=( it.data ))==null?'':__t)+
'</div>';
return __p;
});
cache.put("/options/templates/option.html", function(it){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<div class="ellipsis selected">'+
((__t=( it.meta.name ))==null?'':_.escape(__t))+
'</div>';
return __p;
});
cache.put("/options/templates/script.html", function(it){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<img class="script-icon" src="'+
((__t=( it._icon ))==null?'':_.escape(__t))+
'"> <div class="script-version pull-right"> '+
((__t=( it.meta.version ? 'v' + it.meta.version : '' ))==null?'':_.escape(__t))+
' </div> <div class="script-author ellipsis pull-right" title="'+
((__t=( it.meta.author || '' ))==null?'':_.escape(__t))+
'"> '+
((__t=( it.author ))==null?'':__t)+
' </div> <div class="script-info"> <a class="script-name ellipsis" target="_blank" '+
((__t=( it.homepageURL ? 'href="' + _.escape(it.homepageURL)  + '"' : '' ))==null?'':__t)+
'>'+
((__t=( it.custom.name || it.getLocaleString('name') ))==null?'':_.escape(__t))+
'</a> <a class="script-support '+
((__t=( it.meta.supportURL ? '' : 'hide' ))==null?'':__t)+
'" target="_blank" href="'+
((__t=( it.meta.supportURL ))==null?'':_.escape(__t))+
'"><svg class="icon"><use xlink:href="#question"/></svg></a> </div> <p class="script-desc ellipsis"> '+
((__t=( it.custom.description || it.getLocaleString('description') ))==null?'':_.escape(__t))+
' </p> <div class="buttons"> <button data-id="edit" data-i18n="buttonEdit"></button> <button data-id="enable">'+
((__t=(
    it.enabled ? _.i18n('buttonDisable') : _.i18n('buttonEnable')
  ))==null?'':__t)+
'</button> <button data-id="remove" data-i18n="buttonRemove"></button> '+
((__t=( it.canUpdate ?
  '<button data-id=update data-i18n="buttonUpdate"' + (
    it.checking ? ' disabled' : ''
  ) + '></button>'
  : '' ))==null?'':__t)+
' <span data-id="message">'+
((__t=( it.message ))==null?'':__t)+
'</span> </div>';
return __p;
});
cache.put("/options/templates/tab-about.html", function(it){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<h1> <span data-i18n="labelAbout"></span> <small>v'+
((__t=( it.version ))==null?'':__t)+
'</small> </h1> <div class="line" data-i18n="extDescription"></div> <div class="line"> <label data-i18n="labelRelated"></label> <span data-i18n="anchorSupportPage"></span> | <a href="https://gerald.top/~donate" target="_blank" data-i18n="labelDonate"></a> | <a href="https://github.com/violentmonkey/violentmonkey-oex/issues" target="_blank" data-i18n="labelFeedback"></a> </div> <div class="line"> <label data-i18n="labelAuthor"></label> <span data-i18n="anchorAuthor"></span> </div> <div class="line"> <label data-i18n="labelTranslator"></label> <span data-i18n="anchorTranslator"></span> </div> <div class="line"> <label data-i18n="labelCurrentLang"></label> <span id="currentLang">'+
((__t=( navigator.language ))==null?'':__t)+
'</span> | <a href="https://violentmonkey.github.io/localization.html#oex" target="_blank"> Help with translation </a> </div>';
return __p;
});
cache.put("/options/templates/tab-installed.html", function(it){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<header> <button id="bNew" data-i18n="buttonNew"></button> <button id="bUpdate" data-i18n="buttonUpdateAll"></button> <button id="bURL" data-i18n="buttonInstallFromURL"></button> <div class="pull-right"> <a href="https://greasyfork.org/scripts" target="_blank" data-i18n="anchorGetMoreScripts"></a> </div> </header> <div class="backdrop"><div></div></div> <div class="scripts"></div>';
return __p;
});
cache.put("/options/templates/tab-settings.html", function(it){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<h1 data-i18n="labelSettings"></h1> <label class="line"> <input id="cUpdate" type="checkbox" data-check="autoUpdate" '+
((__t=( it.autoUpdate ? 'checked' : '' ))==null?'':__t)+
'> <span data-i18n="labelAutoUpdate"></span> </label> <fieldset class="title"> <legend data-i18n="labelDataImport"></legend> <div id="groupImport"> <button id="bImport" data-i18n="buttonImportData"></button> <input type="file" id="bImportHelper"> </div> <button id="bVacuum" data-i18n="buttonVacuum" title="'+
((__t=( _.i18n('hintVacuum') ))==null?'':_.escape(__t))+
'"></button> </fieldset> <fieldset class="title"> <legend data-i18n="labelDataExport"></legend> <b data-i18n="labelScriptsToExport"></b> <label> <input id="cbValues" type="checkbox" data-check="exportValues" '+
((__t=( it.exportValues ? 'checked' : '' ))==null?'':__t)+
'> <span data-i18n="labelExportScriptData"></span> </label> <div class="export-list"></div> <button id="bSelect" data-i18n="buttonAllNone"></button> <button id="bExport" data-i18n="buttonExportData"></button> </fieldset>';
return __p;
});
cache.put("/popup/templates/menu.html", function(it){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<div class="menu"></div> '+
((__t=( it.hasSep ? '<hr>' : '' ))==null?'':__t)+
' <div class="menu placeholder"></div> <div class="menu"></div>';
return __p;
});
cache.put("/popup/templates/menuitem.html", function(it){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+=''+
((__t=( it.onClickDetail ? '<div class="menu-item-detail">...</div>' : '' ))==null?'':__t)+
' <div class="menu-item-label"> <svg class="icon"><use xlink:href="#'+
((__t=( it.symbol ))==null?'':__t)+
'"/></svg> '+
((__t=( it.name ))==null?'':_.escape(__t))+
' </div>';
return __p;
});

});
